// 
// robert twomey's
// speech recognition final project dxarts 463, spring 2011
//
// uses pocket-sphinx for speech recognition
// supercollider to handle mic input and sound routing
// send results to receipt printer (through processing via OSC)
//
// usage:
//
// 1. run the server setup
// 2. run the main recognition loop
//
// more info here: http://wiki.roberttwomey.com/Sound463
//
// http://svn.roberttwomey.com/supercollider/speechtool.scd
//
// rtwomey@u.washington.edu


// ==== server setup ==== //

(
var resamplepath, sphinxpath, resultslog;
var ip, port;
var maxrecord;

// paths + files
resamplepath="/Users/rtwomey/code/supercollider/binaries/"; // sndfile-resample
sphinxpath="/Users/rtwomey/code/supercollider/binaries/"; // pocket-sphinx 
resultslog="/Users/rtwomey/Sounds/SpeechFiles/logs/"++Date.getDate.stamp++".txt"; // ASR results log

// OSC setup to communicate results to processing
ip="127.0.0.1"; 
port=12000;

s = Server.internal;

// allocate record buffer
maxrecord=60.0; // seconds
b = Buffer.alloc(s, 44100 * maxrecord, 1);

// create ASR log file
f = File(resultslog, "w");

// create the NetAddr to send OSC messages
n = NetAddr.new(ip, port); 

// add executable paths
"PATH".setenv("$PATH:"++resamplepath++":"++sphinxpath);

// post startup info to screen
"==== setup ====".postln;
("max record time: "++maxrecord++" seconds").postln;
("sndfile-resample path: "++resamplepath).postln;
("pocketsphinx path: "++sphinxpath).postln;
("logging results to: "++resultslog).postln;
("sending osc messages to: "++ip++":"++port).postln;

" ".postln;
"done.".postln;
" ".postln;
)


// ==== main recognition stuff ==== //


(

// timing parameters and threshold levels //
var dur = 0.1;
var ondur=dur;
var offdur=dur*2;
var hi = 0.01;
var lo = hi-0.0091;
var result;

// sound detection and record buffer stuff //
{
	var in, speechSeg;
	var timer;
	var rate;
	
	rate = (Server.default.sampleRate / Server.default.options.blockSize).reciprocal;
	
	// microphone input signal conditioning
	in = SoundIn.ar(1);
	in = LeakDC.ar(in);
	in = Limiter.ar(in, -3.dbamp);

	// simple speech segmenter:
	//   goes high (1.0) when average sound is above threshold
	//   drops back (0.0) down when average sound drops
	speechSeg=SetResetFF.ar(
		Schmidt.ar(RunningSum.rms(in, ondur*44100), lo, hi),
		1.0-Schmidt.ar(RunningSum.rms(in, offdur*44100), lo, lo)
	).scope;

	// record sound with slight delay
	RecordBuf.ar(DelayN.ar(in, ondur, ondur), b, trigger: speechSeg);

	// time length of speech segment
	timer = Phasor.kr(speechSeg, rate, 0, 10000);

//	// send an "utterance_start" message when speech starts
//	SendReply.ar(speechSeg*EnvGen.kr(Env([0, 0, 1], [0.1, 0.1])), 'utterance_start', 0, 1905);
	
	// send the "recognize_me" message slightly after speech is done
	SendReply.ar(TDelay.ar((1.0-speechSeg)*EnvGen.kr(Env([0, 0, 1], [0.1, 0.1])), offdur), 'recognize_me', timer, 1906);
	
}.play(s);


// file output, conversion, and speech recognition stuff //
//
//p.remove; p = OSCresponder(nil, 'utterance_start', { |t, r, msg|
//		"Sound On".postln;
//	}).add;


o.remove; o = OSCresponder(nil, 'recognize_me', { |t, r, msg|
	
	Routine.run({
		var cond, filename, length, timestamp;

		cond = Condition.new;
		// echo osc response to screen (with timing)
		//["result: "+t.round(0.01), msg[3]].postln;
		
		// get length of buffer to write out (seconds)
		length = (msg[3]);
		
		// write time-stamped audio file to sounds directory and wait for done
		timestamp=Date.getDate.stamp;
		filename=("~/Sounds/SpeechFiles/"++timestamp++".wav").standardizePath;
		
		//("Writing: "++timestamp++".wav").postln;
		b.write(filename, "wav", "int16", (length*44100), 0);
		s.sync(cond); 
	
		("Starting ASR: "++timestamp).postln;

		// do sample rate conversion then speech recognition
		("sndfile-resample -to 16000 "
			++filename
			++" /Users/rtwomey/Sounds/resampled.wav"
			).unixCmd({ result =  ("pocketsphinx-osx-continuous"

					// uncomment one of the following to specify language model or grammar
					
					// grammar-based
					+"-jsgf /Users/rtwomey/code/supercollider/data/grammars/languageworld.jsgf"
					
					// script-based grammar (unique words)
//					+"-jsgf /Users/rtwomey/code/supercollider/data/grammars/script.gram"
					
					// MMPI2-based SLM - GOOD
//					+"-dict /Users/rtwomey/code/supercollider/data/models/mmpi2_online/9316.dic"
//					+"-lm /Users/rtwomey/code/supercollider/data/models/mmpi2_online/9316.lm"

					// script-based SLM
//					+"-dict /Users/rtwomey/code/supercollider/data/models/script/8716.dic"
//					+"-lm /Users/rtwomey/code/supercollider/data/models/script/8716.lm"

					// MMPI2-based SLM - not so hot, trying to train locally
//					+"-dict /Users/rtwomey/code/supercollider/data/models/mmpi2_local/2971.dict"
//					+"-lm /Users/rtwomey/code/supercollider/data/models/mmpi2_local/mmpi2.arpa"

					// observatory SLM - descriptions of a few objects in the observatory
//					+"-dict /Users/rtwomey/code/supercollider/data/models/observatory/3239.dic"
//					+"-lm /Users/rtwomey/code/supercollider/data/models/observatory/3239.lm"

					+"-infile /Users/rtwomey/Sounds/resampled.wav".standardizePath
					
					).unixCmdGetStdOut; // speech recognition
					
					result.postln; // echo ASR results to screen
					result = result.copyRange(12, result.size); // clean up string
					n.sendMsg("/printme", result); // send to processing
					
					f.write(timestamp+result); // save to log file
					
					 }
				, false);
				
		//"-jsgf /Users/rtwomey/code/supercollider/pocketsphinx-osx/build/Debug/solipsist.gram"
						
		})

	}).add;

CmdPeriod.doOnce {
	
	// form-feed 8 lines
	d=27.asAscii ++ 100.asAscii ++ 8.asAscii;
	n.sendMsg("/printme", d);

	// paper cut
	d=""++27.asAscii ++ 'i';
	n.sendMsg("/printme", d);

	// remove ocsresponder
	o.remove;
	
	// close text log file
	f.close;
};

"running.".postln;
"".postln;

)

